# -*- coding: utf-8 -*-


def time_equivalence_pd6688(q_fk, delta_1, m, k_b, H, A_f, A_vh, A_vv, H_v, delta_h):
    import numpy as np

    q_fd = q_fk * delta_1 * m
    alpha_v = min([max([A_vv / A_f, 0.025]), 0.25])
    alpha_h = A_vh / A_f
    b_v = 12.5 * (1 + 10 * alpha_v - alpha_v ** 2) if (12.5 * (1 + 10 * alpha_v - alpha_v ** 2)) >= 10 else np.nan
    w_f = ((0.62 + 90 * (0.4 - alpha_v) ** 4) / (1 + b_v * alpha_h))
    w_f = w_f if A_f >= 100 else np.nan

    return q_fd * k_b * w_f * delta_h


def test_time_equivalence_pd6688():

    input_params = {
        "q_fk": 900, "delta_1": 0.61, "m": 1, "k_b": 0.09, "H": 4, "A_f": 856.5, "A_vh": 0, "A_vv": 235.2, "H_v": 4, "delta_h": 2
    }

    res = time_equivalence_pd6688(**input_params)

    print(res)




# -*- coding: utf-8 -*-
import os
import numpy as np

from matplotlib import pyplot as plt
import seaborn as sns
sns.set(style='ticks')

from sfeprapy.dat.steel_carbon import Thermal
from sfeprapy.func.temperature_fires import standard_fire_iso834 as _fire_iso
from sfeprapy.func.temperature_fires import parametric_eurocode1 as _fire_param
from sfeprapy.func.temperature_fires import travelling_fire as _fire_travelling
from sfeprapy.func.temperature_steel_section import protected_steel_eurocode as _steel_heat_transfer

# try:
#     from project.func.temperature_fires import standard_fire_iso834 as _fire_standard
# except ModuleNotFoundError:
#     import sys
#
#     __path_cwf = os.path.realpath(__file__)
#     __dir_project = os.path.dirname(os.path.dirname(__path_cwf))
#
#     sys.path.insert(0, __dir_project)
#
#     from project.func.temperature_fires import standard_fire_iso834 as _fire_standard
#
# from project.func.temperature_fires import standard_fire_iso834 as _fire_iso
# from project.func.temperature_fires import parametric_eurocode1 as _fire_param
# from project.func.temperature_fires import travelling_fire as _fire_travelling
# from project.dat.steel_carbon import Thermal
# from project.func.temperature_steel_section import protected_steel_eurocode as _steel_heat_transfer


def main():
    print('hello world')

    t = np.arange(0, 60*180, 1, dtype=float)
    steel_prop = Thermal()

    # PARAMETRIC FIRE
    inputs_parametric_fire = {
        "A_t": 360,
        "A_f": 100,
        "A_v": 21.6,
        "h_eq": 1,
        "q_fd": 600e6,
        "lambda_": 1,
        "rho": 1,
        "c": 2250000,
        "t_lim": 20 * 60,
        "time_end": 3 * 60 * 60,
        "time_step": 1,
        "time_start": 0,
        "temperature_initial": 273.15
        }
    time_fire_param, temp_fire_param = _fire_param(**inputs_parametric_fire)

    time_steel_param, temp_steel_param, _ = _steel_heat_transfer(
        time=time_fire_param,
        temperature_ambient=temp_fire_param,
        rho_steel=7850,
        c_steel_T=steel_prop.c(),
        area_steel_section=0.017,
        k_protection=0.2,
        rho_protection=800,
        c_protection=1700,
        thickness_protection=0.00938,
        perimeter_protected=2.14,
        is_terminate_peak=False
    )

    # TRAVELLING FIRE

    inputs_travelling_fire = {
        "T_0": 273.15,
        "q_fd": 600e6,
        "RHRf": 0.15e6,
        "l": 25,
        "w": 17.4,
        "s": 0.012,
        "h_s": 4.5,
        "l_s": 20,
        "time_step": 1,
        "time_ubound": 22080
    }
    time_fire_travel, temp_fire_travel, _ = _fire_travelling(**inputs_travelling_fire)

    time_steel_travel, temp_steel_travel, _ = _steel_heat_transfer(
        time=time_fire_travel,
        temperature_ambient=temp_fire_travel,
        rho_steel=7850,
        c_steel_T=steel_prop.c(),
        area_steel_section=0.017,
        k_protection=0.2,
        rho_protection=800,
        c_protection=1700,
        thickness_protection=0.01007,
        perimeter_protected=2.14,
        is_terminate_peak=False
    )

    # MAKE STEEL TEMPERATURE FOR PARAMETRIC FIRE


    # print(np.max(temp_steel_param)-273.15)
    # plt.plot(time_fire_param/60, temp_fire_param)
    # plt.plot(time_steel_param/60, temp_steel_param)
    # plt.show()

    # MAKE STEEL TEMPERATURE FOR TRAVELLING FIRE
    # print(np.max(temp_steel_travel)-273.15)
    # plt.plot(time_fire_travel/60, temp_fire_travel)
    # plt.plot(time_steel_travel/60, temp_steel_travel)
    # plt.show()

    # MAKE ISO 834 FIRE
    time_fire_iso, temp_fire_iso = _fire_iso(np.arange(0, 21601, 1), 20 + 273.15)

    time_steel_iso_param, temp_steel_iso_param, _ = _steel_heat_transfer(
        time=time_fire_iso,
        temperature_ambient=temp_fire_iso,
        rho_steel=7850,
        c_steel_T=steel_prop.c(),
        area_steel_section=0.017,
        k_protection=0.2,
        rho_protection=800,
        c_protection=1700,
        thickness_protection=0.00938,
        perimeter_protected=2.14,
        is_terminate_peak=False
    )

    time_steel_iso_travel, temp_steel_iso_travel, _ = _steel_heat_transfer(
        time=time_fire_iso,
        temperature_ambient=temp_fire_iso,
        rho_steel=7850,
        c_steel_T=steel_prop.c(),
        area_steel_section=0.017,
        k_protection=0.2,
        rho_protection=800,
        c_protection=1700,
        thickness_protection=0.01007,
        perimeter_protected=2.14,
        is_terminate_peak=False
    )

    # fig, ax = plt.subplots(figsize=(2.5, 2))
    fig, ax = plt.subplots(figsize=(3.94, 2.76))

    def mini_plot(fig_, ax_, x, y, fig_name, xlabel='Time', ylabel='Temperature', xlim=(0, 120), ylim=(0, 1400)):
        ax_.plot(x, y)
        ax_.set_xlabel(xlabel)
        ax_.set_ylabel(ylabel)
        ax_.set_xlim(xlim)
        ax_.set_ylim(ylim)
        ax_.set_xticks([])
        ax_.set_yticks([])
        fig_.tight_layout()
        fig_.savefig(fig_name, ppi=300)
        ax_.cla()

    def mini_plot2(fig_, ax_, x, y, fig_name, xlabel='Time', ylabel='Temperature', xlim=(0, 120), ylim=(0, 1400)):
        for i, xx in enumerate(x):
            yy = y[i]
            ax_.plot(xx, yy)
        ax_.set_xlabel(xlabel)
        ax_.set_ylabel(ylabel)
        ax_.set_xlim(xlim)
        ax_.set_ylim(ylim)
        ax_.set_xticks([])
        ax_.set_yticks([])
        fig_.tight_layout()
        fig_.savefig(fig_name, ppi=300)
        ax_.cla()

    def mini_plot3(fig_, ax_, x, y, legend, fig_name, xlabel='Time', ylabel='Temperature', xlim=(0, 120), ylim=(0, 1400)):
        for i, xx in enumerate(x):
            yy = y[i]
            l = legend[i]
            ax_.plot(xx, yy, lable=l)
        ax_.set_xlabel(xlabel)
        ax_.set_ylabel(ylabel)
        ax_.set_xlim(xlim)
        ax_.set_ylim(ylim)
        ax_.set_xticks([])
        ax_.set_yticks([])
        fig_.tight_layout()
        fig_.savefig(fig_name, ppi=300)
        ax_.cla()

    mini_plot(fig, ax, time_fire_param/60., temp_fire_param - 273.15, 'fire_param.png')

    mini_plot(fig, ax, time_fire_iso/60., temp_fire_iso - 273.15, 'fire_iso.png')

    mini_plot(fig, ax, time_fire_travel/60., temp_fire_travel - 273.15, 'fire_travel.png')

    # mini_plot2(fig, ax,
    #            [time_fire_param/60., time_steel_param/60.],
    #            [temp_fire_param - 273.15, temp_steel_param-273.5],
    #            'steel_param.png')

    # mini_plot2(fig, ax,
    #           [time_fire_travel/60., time_steel_travel/60.],
    #           [temp_fire_travel - 273.15, temp_steel_travel-273.5],
    #           'steel_travel.png')

    # mini_plot2(fig, ax,
    #            [time_fire_iso/60., time_steel_iso_travel/60.],
    #            [temp_fire_iso - 273.15, temp_steel_iso_travel-273.5],
    #            'steel_iso.png')

    # mini_plot2(fig, ax,
    #            [time_fire_iso/60., time_steel_iso_travel/60.],
    #            [temp_fire_iso - 273.15, temp_steel_iso_travel-273.15],
    #            'steel_iso_travel.png')

    # mini_plot2(fig, ax,
    #            [time_fire_travel / 60., time_steel_travel / 60.],
    #            [temp_steel_travel - 273.15, temp_steel_iso_travel - 273.15],
    #            'steel_iso_travel.png')

    mini_plot2(fig, ax,
               [time_fire_param/60., time_fire_travel/60.],
               [temp_fire_param - 273.15, temp_fire_travel - 273.15],
               ['Parametric fire', 'Travelling fire'],
               'fire_param_and_travel.png')

    # plt.plot(time_fire_iso/60, temp_fire_iso)
    # plt.plot(time_steel_iso_param/60, temp_steel_iso_param, '--')
    # plt.plot(time_fire_param/60, temp_fire_param)
    # plt.plot(time_steel_param/60, temp_steel_param, '--')
    # plt.show()

    # plt.plot(time_fire_iso/60, temp_fire_iso)
    # plt.plot(time_steel_iso_travel/60, temp_steel_iso_travel, '--')
    # plt.plot(time_fire_travel/60, temp_fire_travel)
    # plt.plot(time_steel_travel/60, temp_steel_travel, '--')
    # plt.show()

    # MAKE STEEL TEMPERATURE FOR ISO 834 FIRE



# -*- coding: utf-8 -*-
import numpy as np


def parametric_eurocode1(A_t, A_f, A_v, h_eq, q_fd, lambda_, rho, c, t_lim, time_end=7200, time_step=1, time_start=0, time_padding = (0, 0),temperature_initial=293.15, is_more_return=False):
    """Function Description: (SI UNITS ONLY)
    This function calculates the time-temperature curve according to Eurocode 1 part 1-2, Appendix A.
    :param A_t:
    :param A_f:
    :param A_v:
    :param h_eq:
    :param q_fd:
    :param lambda_:
    :param rho:
    :param c:
    :param t_lim:
    :param time_end:
    :param time_step:
    :param time_start:
    :param time_padding:
    :param temperature_initial:
    :return t:
    :return T_g:
    """
    # Reference: Eurocode 1991-1-2; Jean-Marc Franssen, Paulo Vila Real (2010) - Fire Design of Steel Structures

    # UNITS: SI -> Equations
    q_fd /= 1e6  # [J/m2] -> [MJ/m2]
    t_lim /= 3600  # [s] -> [hr]
    time_end /= 3600  # [s] -> [hr]
    time_step /= 3600  # [s] -> [hr]
    time_start /= 3600  # [s] -> [hr]
    temperature_initial -= 273.15  # [K] -> [C]

    # ACQUIRING REQUIRED VARIABLES
    t = np.arange(time_start, time_end, time_step, dtype=float)

    b = (lambda_ * rho * c) ** 0.5
    O = A_v * h_eq**0.5 / A_t
    q_td = q_fd * A_f / A_t
    Gamma = ((O/0.04)/(b/1160))**2

    t_max = 0.0002*q_td/O

    # check criteria
    if not 50 <= q_td <= 1000:
        print("q_td = {:4.1f} not in range [50, 1000]".format(q_td))

    # CALCULATION
    def _temperature_heating(t_star, temperature_initial):
        # eq. 3.12
        T_g = 1325 * (1 - 0.324*np.exp(-0.2*t_star) - 0.204*np.exp(-1.7*t_star) - 0.472*np.exp(-19*t_star))
        T_g += temperature_initial
        return T_g

    def _temperature_cooling_vent(t_star_max, T_max, t_star):  # ventilation controlled
        # eq. 3.16
        if t_star_max <= 0.5:
            T_g = T_max - 625 * (t_star - t_star_max)
        elif 0.5 < t_star_max < 2.0:
            T_g = T_max - 250 * (3 - t_star_max) * (t_star - t_star_max)
        elif 2.0 <= t_star_max:
            T_g = T_max - 250 * (t_star - t_star_max)
        else: T_g = np.nan
        return T_g

    def _temperature_cooling_fuel(t_star_max, T_max, t_star, Gamma, t_lim):  # fuel controlled
        # eq. 3.22
        if t_star_max <= 0.5:
            T_g = T_max - 625 * (t_star - Gamma * t_lim)
        elif 0.5 < t_star_max < 2.0:
            T_g = T_max - 250 * (3 - t_star_max) * (t_star - Gamma * t_lim)
        elif 2.0 <= t_star_max:
            T_g = T_max - 250 * (t_star - Gamma * t_lim)
        else: T_g = np.nan
        return T_g

    def _variables(t, Gamma, t_max):
        t_star = Gamma * t
        t_star_max = Gamma * t_max
        return t_star, t_star_max

    def _variables_2(t, t_lim, q_td, b, O):
        O_lim = 0.0001 * q_td / t_lim
        Gamma_lim = ((O_lim/0.04)/(b/1160))**2

        if O > 0.04 and q_td < 75 and b < 1160:
            k = 1 + ((O-0.04)/(0.04)) * ((q_td-75)/(75)) * ((1160-b)/(1160))
            Gamma_lim *= k

        t_star_ = Gamma_lim * t
        t_star_max_ = Gamma_lim * t_lim
        return t_star_, t_star_max_

    t_star, t_star_max = _variables(t, Gamma, t_max)

    if t_max >= t_lim:  # ventilation controlled fire
        T_max = _temperature_heating(t_star_max, temperature_initial)
        T_heating_g = _temperature_heating(Gamma * t, temperature_initial)
        T_cooling_g = _temperature_cooling_vent(t_star_max, T_max, t_star)
        fire_type = "ventilation controlled"
    else:  # fuel controlled fire
        t_star_, t_star_max_ = _variables_2(t, t_lim, q_td, b, O)
        T_max = _temperature_heating(t_star_max_, temperature_initial)
        T_heating_g = _temperature_heating(t_star_, temperature_initial)
        T_cooling_g = _temperature_cooling_fuel(t_star_max, T_max, t_star, Gamma, t_lim)
        fire_type = "fuel controlled"

    T_g = np.minimum(T_heating_g, T_cooling_g)
    T_g[T_g < temperature_initial] = temperature_initial

    data_all = {"fire_type": fire_type}

    # UNITS: Eq. -> SI
    t *= 3600
    T_g += 273.15

    if is_more_return:
        return t, T_g, data_all
    else:
        return t, T_g


def standard_fire_iso834(
        time,
        temperature_initial
):

    # INPUTS CHECK
    time = np.array(time, dtype=float)
    time[time < 0] = np.nan

    # SI UNITS -> EQUATION UNITS
    temperature_initial -= 273.15  # [K] -> [C]
    time /= 60.  # [s] - [min]

    # CALCULATE TEMPERATURE BASED ON GIVEN TIME
    temperature = 345. * np.log10(time * 8. + 1.) + temperature_initial
    temperature[temperature == np.nan] = temperature_initial

    # EQUATION UNITS -> SI UNITS
    time *= 60.  # [min] -> [s]
    temperature += 273.15  # [C] -> [K]

    return time, temperature


def standard_fire_astm_e119(
        time,
        temperature_ambient
):
    time /= 1200.  # convert from seconds to hours
    temperature_ambient -= 273.15  # convert temperature from kelvin to celcius
    temperature = 750 * (1 - np.exp(-3.79553 * np.sqrt(time))) + 170.41 * np.sqrt(time) + temperature_ambient
    return temperature + 273.15  # convert from celsius to kelvin (SI unit)


def hydrocarbon_eurocode(
        time,
        temperature_initial
):
    time /= 1200.  # convert time unit from second to hour
    temperature_initial -= 273.15  # convert temperature from kelvin to celsius
    temperature = 1080 * (1 - 0.325 * np.exp(-0.167 * time) - 0.675 * np.exp(-2.5 * time)) + temperature_initial
    return temperature + 273.15


def external_fire_eurocode(
        time,
        temperature_initial
):
    time /= 1200.  # convert time from seconds to hours
    temperature_initial -= 273.15  # convert ambient temperature from kelvin to celsius
    temperature = 660 * (1 - 0.687 * np.exp(-0.32 * time) - 0.313 * np.exp(-3.8 * time)) + temperature_initial
    return temperature + 273.15  # convert temperature from celsius to kelvin


def travelling_fire(
        T_0,
        q_fd,
        RHRf,
        l,
        w,
        s,
        # A_v,
        # h_eq,
        h_s,
        l_s,
        temperature_max=1050,
        time_ubound=10800,
        time_step=1):
    """
    :param T_0: [float][K] Initial temperature.
    :param q_fd: [float][J m2] Fire load density.
    :param RHRf: [float][W m2] Heat release rate density
    :param l: [float][m] Compartment length
    :param w: [float][m] Compartment width
    :param s: [float][m/s] Fire spread speed
    # :param A_v: [float][m2] Ventilation area
    # :param h_eq: [float][m] Weighted ventilation height
    :param h_s: [float][m] Vertical distance between element to fuel bed.
    :param l_s: [float][m] Horizontal distance between element to fire front.
    :param time_ubound: [float][s] Maximum time for the curve.
    :param time_step: [float][s] Static time step.
    :return time: [ndarray][s] An array representing time incorporating 'temperature'.
    :return temperature: [ndarray][K] An array representing temperature incorporating 'time'.
    """

    # SETTINGS
    time_lbound = 0

    # UNIT CONVERSION TO FIT EQUATIONS
    T_0 -= 273.15
    q_fd /= 1e6
    RHRf /= 1e6

    # MAKE TIME ARRAY
    time = np.arange(time_lbound, time_ubound+time_step, time_step)

    # fire_load_density_MJm2=900
    # heat_release_rate_density_MWm2=0.15
    # length_compartment_m=150
    # width_compartment_m=17.4
    # fire_spread_rate_ms=0.012
    # area_ventilation_m2=190
    # height_ventilation_opening_m=3.3
    # height_fuel_to_element_m=3.5
    # length_element_to_fire_origin_m=105

    # workout burning time etc.
    t_burn = max([q_fd / RHRf, 900.])
    t_decay = max([t_burn, l / s])
    t_lim = min([t_burn, l / s])

    # reduce resolution to fit time step for t_burn, t_decay, t_lim
    t_decay_ = round(t_decay/time_step, 0) * time_step
    t_lim_ = round(t_lim/time_step, 0) * time_step
    if t_decay_ == t_lim_: t_lim_ -= time_step

    # workout the heat release rate ARRAY (corrected with time)
    Q_growth = (RHRf * w * s * time) * (time < t_lim_)
    Q_peak = min([RHRf * w * s * t_burn, RHRf * w * l]) * (time >= t_lim_) * (time <= t_decay_)
    Q_decay = (max(Q_peak) - (time-t_decay_) * w * s * RHRf) * (time > t_decay_)
    Q_decay[Q_decay < 0] = 0
    Q = (Q_growth + Q_peak + Q_decay) * 1000.

    # workout the distance between fire_curve midian to the structural element r
    l_fire_front = s * time
    l_fire_front[l_fire_front < 0] = 0.
    l_fire_front[l_fire_front > l] = l
    l_fire_end = s * (time - t_lim)
    l_fire_end[l_fire_end < 0] = 0.
    l_fire_end[l_fire_end > l] = l
    l_fire_median = (l_fire_front + l_fire_end) / 2.
    r = np.absolute(l_s - l_fire_median)
    r[r == 0] = 0.001  # will cause crash if r = 0

    # workout the far field temperature of gas T_g
    T_g1 = (5.38 * np.power(Q / r, 2 / 3) / h_s) * ((r/h_s) > 0.18)
    T_g2 = (16.9 * np.power(Q, 2 / 3) / np.power(h_s, 5/3)) * ((r/h_s) <= 0.18)
    T_g = T_g1 + T_g2 + T_0

    T_g[T_g >= temperature_max] = temperature_max

    # UNIT CONVERSION TO FIT OUTPUT (SI)
    T_g += 273.15  # C -> K
    Q *= 10e6  # MJ -> J

    temperature = T_g

    data_trivial = {
        "heat release [J]": Q,
        "distance fire to element [m]": r
    }
    return time, temperature, data_trivial


def t_square(time, growth_factor, cap_hrr=0, cap_hrr_to_time=0, terminate_at_peak=False):
    if isinstance(growth_factor, str):
        growth_factor = str.lower(growth_factor)
        growth_dict = {"slow": 0.0029,
                       "medium": 0.0117,
                       "fast": 0.0469,
                       "ultra-fast": 0.1876}
        try:
            growth_factor = growth_dict[growth_factor]
        except KeyError:
            err_msg = "{} should be one of the following if not a number: {}"
            err_msg = err_msg.format("growth_factor", ", ".join(list(growth_dict.keys())))
            raise ValueError(err_msg)

    heat_release_rate = growth_factor * time ** 2 * 1000

    # cap hrr
    if cap_hrr:
        heat_release_rate[heat_release_rate>cap_hrr] = cap_hrr

    if cap_hrr_to_time:
        heat_release_rate[time>cap_hrr_to_time] = -1
        heat_release_rate[heat_release_rate==-1] = max(heat_release_rate)

    return heat_release_rate


if __name__ == "__main__":
    time = np.arange(0, 2*60*60, 30)
    temperature_cap = 450

    time, temperature = standard_fire_iso834(time=np.arange(0, 2*60*60+1), temperature_initial=20+273.15)

    temperature -= 273.15
    temperature[temperature > temperature_cap] = temperature_cap

    import seaborn as sb
    import matplotlib.pyplot as plt

    sb.scatterplot(time, temperature)
    plt.show()

    time_temperature = list(zip(time, temperature))

    print(time_temperature)

    time_temperature = ['{:.0f},{:.1f}'.format(*i) for i in time_temperature]

    with open('o.txt', 'w+') as f:
        f.write('\n'.join(time_temperature))
    # t_square()



from PySide2 import QtCore, QtGui, QtWidgets


class Dialog(QtWidgets.QDialog):
    NumGridRows = 3
    NumButtons = 4

    def __init__(self):
        super(Dialog, self).__init__()

        self.createMenu()
        self.createGridGroupBox()
        self.createFormGroupBox()

        bigEditor = QtWidgets.QTextEdit()
        bigEditor.setPlainText("This widget takes up all the remaining space "
                               "in the top-level layout.")

        buttonBox = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok | QtWidgets.QDialogButtonBox.Cancel)

        buttonBox.accepted.connect(self.accept)
        buttonBox.rejected.connect(self.reject)

        mainLayout = QtWidgets.QVBoxLayout()
        mainLayout.setMenuBar(self.menuBar)
        # mainLayout.addWidget(self.horizontalGroupBox)
        mainLayout.addWidget(self.gridGroupBox)
        mainLayout.addWidget(self.formGroupBox)
        mainLayout.addWidget(bigEditor)
        mainLayout.addWidget(buttonBox)
        self.setLayout(mainLayout)

        self.setWindowTitle("Basic Layouts")

    def createMenu(self):
        self.menuBar = QtWidgets.QMenuBar()

        self.fileMenu = QtWidgets.QMenu("&File", self)
        self.exitAction = self.fileMenu.addAction("E&xit")
        self.menuBar.addMenu(self.fileMenu)

        self.exitAction.triggered.connect(self.accept)


    def createGridGroupBox(self):
        self.gridGroupBox = QtWidgets.QGroupBox("Grid layout")
        layout = QtWidgets.QGridLayout()

        r1_label = QtWidgets.QLabel("Problems")
        r1_edit = QtWidgets.QLineEdit()
        r1_button = QtWidgets.QPushButton("Folder")
        layout.addWidget(r1_label, 0, 0)
        layout.addWidget(r1_edit, 0, 1)
        layout.addWidget(r1_button, 0, 2, 1, 2)

        r2_label = QtWidgets.QLabel("Preferences")
        r2_edit = QtWidgets.QLineEdit()

        def r2_act_button_1(): r2_edit.setText(self.get_file_path(filter="YAML (*.yaml, *.txt)"))

        r2_button_1 = QtWidgets.QPushButton("File")
        r2_button_1.setFixedWidth(45)
        r2_button_1.clicked.connect(r2_act_button_1)
        r2_button_2 = QtWidgets.QPushButton("Editor")
        r2_button_2.setFixedWidth(45)
        layout.addWidget(r2_label, 1, 0)
        layout.addWidget(r2_edit, 1, 1)
        layout.addWidget(r2_button_1, 1, 2)
        layout.addWidget(r2_button_2, 1, 3)

        r3_label = QtWidgets.QLabel("Inputs")
        r3_edit = QtWidgets.QLineEdit()

        def r3_act_button_1(): r3_edit.setText(self.get_file_path(filter="YAML (*.yaml, *.txt)"))

        r3_button_1 = QtWidgets.QPushButton("File")
        r3_button_1.setFixedWidth(45)
        r3_button_1.clicked.connect(r3_act_button_1)
        r3_button_2 = QtWidgets.QPushButton("Editor")
        r3_button_2.setFixedWidth(45)
        layout.addWidget(r3_label, 4, 0)
        layout.addWidget(r3_edit, 4, 1)
        layout.addWidget(r3_button_1, 4, 2)
        layout.addWidget(r3_button_2, 4, 3)

        r2_label = QtWidgets.QLabel("Settings:")
        line_edit_2 = QtWidgets.QLineEdit()

        self.smallEditor = QtWidgets.QTextEdit()
        self.smallEditor.setPlainText("This widget takes up about two thirds "
                                      "of the grid layout.")

        layout.setColumnStretch(1, 10)
        layout.setColumnStretch(2, 20)
        self.gridGroupBox.setLayout(layout)

    def get_file_path(self, caption=None, dir=None, filter=None):
        my_dialog = QtWidgets.QFileDialog(self)
        file_name, _ = QtWidgets.QFileDialog.getOpenFileName(my_dialog, caption=caption, dir=dir, filter=filter)
        return file_name

    def create_action(self):
        self.openAct = QtWidgets.QAction(QtGui.QIcon(':/images/open.png'),
                                         "&Open...", self, shortcut=QtGui.QKeySequence.Open,
                                         statusTip="Open an existing file", triggered=self.open)

    def createFormGroupBox(self):
        self.formGroupBox = QtWidgets.QGroupBox("Form layout")
        layout = QtWidgets.QFormLayout()
        layout.addRow(QtWidgets.QLabel("Line 1:"), QtWidgets.QLineEdit())
        layout.addRow(QtWidgets.QLabel("Line 2, long text:"), QtWidgets.QComboBox())
        layout.addRow(QtWidgets.QLabel("Line 3:"), QtWidgets.QSpinBox())
        self.formGroupBox.setLayout(layout)


if __name__ == '__main__':
    import sys

    app = QtWidgets.QApplication(sys.argv)
    dialog = Dialog()
    sys.exit(dialog.exec_())





def list_all_files(root_dir):
    import os, copy

    all_filenames = []

    for dirpath, _, filenames in os.walk(root_dir):

        for f in filenames:

            yield os.path.abspath(os.path.join(dirpath, f))

    # print(all_filenames)


if __name__ == '__main__':
    a = list_all_files(r'D:\Dropbox (OFR-UK)\Bicester_team_projects\Enquiries')

    for i in a:
        if len(i) > 200-43:
            print(i)



import tkinter as tk


if __name__ == '__main__':
    master = tk.Tk()

    dict_entries = {
        'item 1': int,
        'item 2': str,
    }

    master.title('Hello World!')

    i = 0
    dict_tk_entry = {}
    for key, val in dict_entries.items():
        tk.Label(master, text=str(key)).grid(row=i)
        dict_tk_entry[key] = tk.Entry(master)
        dict_tk_entry[key].grid(row=i, column=1)
        i += 1

    # tk.Label(master, text="First").grid(row=0)
    # tk.Label(master, text="Second").grid(row=1)

    # e1 = tk.Entry(master)
    # e2 = tk.Entry(master)
    #
    # e1.grid(row=0, column=1)
    # e2.grid(row=1, column=1)

    master.mainloop()


# -*- coding: utf-8 -*-
import csv
import os


def list_to_csv(file_name, a_list):
    with open(file_name, 'wb') as f:
        output_writer = csv.writer(f, dialect='excel')
        output_writer.writerows(a_list)


def list_all_files_with_suffix(directory_str, suffix_str, is_full_dir=True):
    list_files = []

    for f in os.listdir(directory_str):
        if f.endswith(suffix_str):
            if is_full_dir:
                list_files.append(os.path.join(directory_str, f))
            else:
                list_files.append(f)

    return list_files



#!/usr/bin/env python

# from distutils.core import setup

import os
import setuptools

# To use a consistent encoding
from codecs import open

# Get the long description from the README file
with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'README.md')) as file:
    long_description = file.read()

setuptools.setup(
    name='sfeprapy',

    version='0.0.7',

    description='Structural fire safety engineering - probabilistic reliability assessment',

    author='Yan Fu',

    author_email='fuyans@gmail.com',

    url='https://github.com/fsepy/sfeprapy',

    download_url="https://github.com/fsepy/sfeprapy/archive/master.zip",

    keywords=["fire safety", "structural fire engineering"],

    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 3 - Alpha",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],

    long_description='Structural fire safety engineering - probabilistic reliability assessment',

    packages=['sfeprapy', 'sfeprapy.func', 'sfeprapy.dat', 'sfeprapy.cls', 'sfeprapy.mc'],

    # entry_points={'console_scripts': ['sfeprapymc = sfeprapy.mc.__main__:run']},

    install_requires=[
        'matplotlib>=2.2.2',
        'numpy>=1.15.0',
        'pandas>=0.23.3',
        'scipy>=1.1.0',
        'seaborn>=0.9.0', ],

    include_package_data=True,
)



# -*- coding: utf-8 -*-
import copy
import re


def kwargs_from_text(raw_input_text_str):

    # with open(input_file_dir) as input_file:
    #     input_text = input_file.read()

    input_text = copy.copy(raw_input_text_str)

    # remove spaces
    input_text = input_text.replace(" ", "")

    # remove tabs
    input_text = input_text.replace("\t", "")

    # break raw input text to a list
    input_text = re.split('[;\r\n]', input_text)

    # delete comments (anything followed by #)
    input_text = [v.split("#")[0] for v in input_text]

    # delete empty entries
    input_text = [v for v in input_text if v]

    # for each entry in the list (input_text), break up i.e. ["variable_name=1+1"] to [["variable_name"], ["1+1"]]
    input_text = [v.split("=") for v in input_text]

    # analyse for each individual element input and create a list of library
    dict_inputs = {}
    for each_entry in input_text:
        dict_inputs[each_entry[0]] = eval(each_entry[1])
    return dict_inputs


if __name__ == "__main__":
    inputs_string = """# this is a comment
    my_variable_1=2.5;
    my_variable_2 = 3e8  # speed of light [m/2];
    my_dictionary_1 ={"key1":1,"key2":2,"key3":"hello_string"};
    my_string= "hello_world";
    # end"""

    inputs_dict=kwargs_from_text(inputs_string)
    print(inputs_dict)


# __author__ = "RVC"
# __email__= "ruben.vancoile@gmail.com"
# __date__= "2017-11-12"

from copy import deepcopy
from scipy.stats import uniform
from scipy.stats import gumbel_r
from scipy.stats import norm
from scipy.stats import lognorm
from scipy.stats import t

import numpy as np


## inverse CDF function ##
##########################

def Finv_Uniform(r, a, b):
    return uniform.ppf(r, a, b - a)


def Finv_Gumbel(r, m, s):
    scale, loc = p_Gumbel(m, s)

    return gumbel_r.ppf(r, loc, scale)


def Finv_Normal(r, m, s):
    return norm.ppf(r, m, s)


def Finv_Lognormal(r, m, s):
    sln, mln = p_Lognormal(m, s)

    return lognorm.ppf(r, sln, 0, np.exp(mln))


def Finv_t(r, df):
    return t.ppf(r, df)


## CDF function ##
##################

def F_Normal(x, m, s):
    return norm.cdf(x, m, s)


def F_Lognormal(x, m, s):
    sln, mln = p_Lognormal(m, s)

    return lognorm.cdf(x, sln, 0, np.exp(mln))


def F_Gumbel(r, m, s):
    scale, loc = p_Gumbel(m, s)

    return gumbel_r.cdf(r, loc, scale)


## parameter calculation ##
###########################

def p_Lognormal(m, s):
    cov = s / m;

    sln = np.sqrt(np.log(1 + cov ** 2))
    mln = np.log(m) - 1 / 2 * sln ** 2

    return sln, mln


if __name__ == '__main__':
    print(p_Lognormal(0.2, 0.2))


def p_Gumbel(m, s):
    # parameters Gumbel W&S
    alpha = 1.282 / s
    u = m - 0.5772 / alpha

    # parameters Gumbel scipy
    scale = 1 / alpha
    loc = u

    return scale, loc


## 'moment' calculation ##
##########################

def m_Lognormal(mln, sln):
    cov = np.sqrt(np.exp(sln ** 2) - 1)
    m = np.exp(mln + 1 / 2 * sln ** 2)

    return m, m * cov



# -*- coding: utf-8 -*-
"""Resistance to lateral-torsional buckling

This module is created to aid the calculation principles for structural steel in fire situation set out in EC 1,
PART 1-2. More detailed instruction can be found in F. Jean-Marc & R.P. Vila (2010) - Design of Steel Structures, which
the code in this module compliant with. Section 5.4 is particularly followed.
"""
import numpy as np
from sfeprapy.dat.steel_carbon import thermal


def _M_cr(C_1, C_2, I_z, k_z, k_w, I_w, I_t, L, E, G, z_g):
    """
    [Eq. 5.59]
    :param C_1: Constant, found in Table 5.4 or ECCS (2006)
    :param C_2: ECCS (2006) or in Galea Y (2002)
    :param I_z: Second moment of area about the minor axis
    :param k_z: 0.5 for full restraint, 0.7 for one end fixed and one pinned, 1.0 for both pinned
    :param k_w: 0.5 for full restraint, 0.7 for one end fixed and one pinned, 1.0 for both pinned
    :param I_w: Warping constant
    :param I_t: Torsion constant
    :param L:   Beam/column length
    :param E:   Steel modulus
    :param G:   Steel ?
    :param z_g: Factor governed by the load location on the section, see Fig. 5.24
    :return M_cr: The elastic critical moment for a uniform moment
    """
    a = C_1 * (np.pi**2 * E * I_z) / (k_z * L)**2
    b = (k_z / k_w)**2 * (I_w / I_z)
    c = (k_z * L)**2 * G * I_t / np.pi**2 / E / I_z
    d = (C_2 * z_g) ** 2
    e = C_2 * z_g

    M_cr = a * (b+c+d)**0.5 - e

    return M_cr


def _lambda_LT(W_pl_y, f_y, M_cr):
    """
    [Eq. 5.66]
    :param W_pl_y:      Plastic section modulus
    :param f_y:         Yield strength
    :param M_cr:        Critical bending moment, see [Eq. 5.59]
    :return lambda_LT:  Non-dimensional slenderness at normal temperature
    """

    lambda_LT = (W_pl_y * f_y / M_cr)**2

    return lambda_LT


def _lambda_LT_theta_com(lambda_LT, k_y_theta_com, k_E_theta_com):
    """
    [Eq. 5.65]
    :param lambda_LT: Non-dimensional slenderness at normal temperature
    :param k_y_theta_com: Reduction factor for Young's modulus at the maximum steel temp. in the com. flange
    :param k_E_theta_com: Reduction factor for Young's modulus at the maximum steel temp. in the com. flange
    :return lambda_LT_theta_com: Non-dimensional slenderness at elevated temperature
    """
    lambda_LT_theta_com = lambda_LT * (k_y_theta_com / k_E_theta_com)

    return lambda_LT_theta_com

def _alpha(f_y):
    """
    [Eq. 5.64]
    :param f_y: Steel yield strength
    :return alpha: The imperfection factor
    """

    alpha = 0.65 * (235/f_y) ** 0.5

    return alpha

def _phi_LT_theta_com(alpha, lambda_LT_theta_com):
    """
    [Eq. 5.63]
    :param alpha: The imperfection factor
    :param lambda_LT_theta_com: Non-dimensional slenderness at elevated temeperature
    :return phi_LT_theta_com: A variable used in [5.64]
    """

    phi_LT_theta_com = 0.5 * (1 + alpha * lambda_LT_theta_com + lambda_LT_theta_com**2)

    return phi_LT_theta_com


def _chi_LT_fi(phi_LT_theta_com, lambda_LT_theta_com):
    """
    [Eq. 5.62]
    :param phi_LT_theta_com: Refer to [5.63]
    :param lambda_LT_theta_com: Non-dimensional slenderness at elevated temperature
    :return chi_LT_fi: Reduction factor for lateral-torsion buckling in the fire design situation
    """

    chi_LT_fi = (phi_LT_theta_com + (phi_LT_theta_com**2-lambda_LT_theta_com**2))**-0.5

    return chi_LT_fi

def _M_b_fi_t_Rd(chi_LT_fi, W_y, k_y_theta_com, f_y, gamma_M_fi):
    """
    [Eq. 5.61]
    :param chi_LT_fi: Reduction factor for lateral-torsion buckling in the fire design situation
    :param W_y: Sectional modulus (plastic for class 1 steel)
    :param k_y_theta_com: Reduction factor for yield strength at elevated temperature
    :param f_y: Steel yield strength
    :param gamma_M_fi: Partial safety factor
    :return M_b_fi_t_Rd: The resistant lateral torsion bending moment
    """

    M_b_fi_t_Rd = chi_LT_fi * W_y * k_y_theta_com * f_y / gamma_M_fi

    return M_b_fi_t_Rd


if __name__ == "__main__":
    # REQUIRED PARAMETERS
    C_1 = 1.77
    C_2 = 0.
    I_z = 11400.e-8  # m4

    k_z = 1.  # conservative
    k_w = 1.  # conservative
    I_w = 19.3e-6  # m6
    I_t = 514.e-8  # m4
    L = 2.5  # m
    E = 210e9  # Pa
    G = 81e9  # Pa
    z_g = 0.
    theta = 273.15 + 20  # K
    theta = np.arange(0, 550, 50) + 273.15

    W_py_y = 7980e-6  # m3
    W_y = W_py_y
    f_y = 345e6  # Pa

    gamma_M_fi = 1

    # Calculate: The strength reduction factor for steel in fire condition
    k_y_theta_com = thermal("reduction factor for effective yield strength")(theta)
    k_E_theta_com = thermal("reduction factor for the slope of the linear elastic range")(theta)

    # Calculate: The critical bending moment resistance
    M_cr = _M_cr(C_1, C_2, I_z, k_z, k_w, I_w, I_t, L, E, G, z_g)

    # Calculate: The non-dimensional slenderness at normal temperature
    lambda_LT = _lambda_LT(W_py_y, f_y, M_cr)

    # Calculate: The non-dimensional slenderness at elevated temperature
    lambda_LT_theta_com =_lambda_LT_theta_com(lambda_LT, k_y_theta_com, k_E_theta_com)

    # Calculate: The imperfection factor for material
    alpha = _alpha(f_y)

    # Calculate: A variable used for later
    phi_LT_theta_com = _phi_LT_theta_com(alpha, lambda_LT_theta_com)

    # Calculate: The reduction factor for lateral torsion in fire condition
    chi_LT_fi = _chi_LT_fi(phi_LT_theta_com, lambda_LT_theta_com)

    # Calculate: The lateral torsion buckling moment resistance in fire condition
    M_b_fi_t_Rd = _M_b_fi_t_Rd(chi_LT_fi, W_y, k_y_theta_com, f_y, gamma_M_fi)

    # OUTPUTS
    print("The elastic resistance moment for lateral torsion is: {:.2f} MN m.".format(M_cr * 1e-6))
    print("The design lateral-torsional buckling resistance moment is: {:.2f} MN m.".format(M_b_fi_t_Rd * 1e-6))



# -*- coding: utf-8 -*-
import numpy as np

def travelling_fire(
        fire_load_density_MJm2,
        heat_release_rate_density_MWm2,
        length_compartment_m,
        width_compartment_m,
        fire_spread_rate_ms,
        height_fuel_to_element_m,
        length_element_to_fire_origin_m,
        time_start_s,
        time_end_s,
        time_interval_s,
        nft_max_C,
        win_width_m,
        win_height_m,
        open_fract
    ):
    # make time array
    t = np.arange(time_start_s, time_end_s, time_interval_s)

    # re-assign variable names for equation readability
    q_fd = fire_load_density_MJm2
    RHRf = heat_release_rate_density_MWm2
    l = max([length_compartment_m, width_compartment_m])
    w = min([length_compartment_m, width_compartment_m])
    s = fire_spread_rate_ms
    h_s = height_fuel_to_element_m
    l_s = length_element_to_fire_origin_m

    # work out ventilation conditions
    a_v = win_height_m * win_width_m * open_fract
    Qv = 1.75 * a_v * np.sqrt(win_height_m)

    # workout burning time etc.
    t_burn = max([q_fd / RHRf, 900.])
    t_decay = max([t_burn, l / s])
    t_lim = min([t_burn, l / s])

    # reduce resolution to fit time step for t_burn, t_decay, t_lim
    t_decay_ = round(t_decay / time_interval_s, 0) * time_interval_s
    t_lim_ = round(t_lim / time_interval_s, 0) * time_interval_s
    if t_decay_ == t_lim_: t_lim_ -= time_interval_s

    # workout the heat release rate ARRAY (corrected with time)
    Q_growth = (RHRf * w * s * t) * (t < t_lim_)
    Q_peak = min([RHRf * w * s * t_burn, RHRf * w * l]) * (t >= t_lim_) * (t <= t_decay_)
    Q_decay = (max(Q_peak) - (t - t_decay_) * w * s * RHRf) * (t > t_decay_)
    Q_decay[Q_decay < 0] = 0
    Q = (Q_growth + Q_peak + Q_decay) * 1000.

    # workout the distance between fire midian to the structural element r
    l_fire_front = s * t
    l_fire_front[l_fire_front < 0] = 0.
    l_fire_front[l_fire_front > l] = l
    l_fire_end = s * (t - t_lim)
    l_fire_end[l_fire_end < 0] = 0.
    l_fire_end[l_fire_end > l] = l
    l_fire_median = (l_fire_front + l_fire_end) / 2.
    r = np.absolute(l_s - l_fire_median)

    # workout the far field temperature of gas T_g

    T_g1 = (5.38 * np.power(Q / r, 2 / 3) / h_s) * (r / h_s > 0.18).astype(int)
    T_g2 = (16.9 * np.power(Q, 2 / 3) / np.power(h_s, 5/3)) * (r/h_s <= 0.18).astype(int)
    T_g = T_g1 + T_g2 + 20.
    T_g[T_g>=nft_max_C] = nft_max_C

    return t, T_g, Q, r
